# Online Learning Data Warehouse for Python

This package provides a SQLAlchemy ORM interface to data found in the U-M Online Learning Data Warehouse as represented on EDWPROD.

## Installation in a project

An index-url can be used to install the library, with `pip`, such as:

```shell
$ pip install --index-url https://packages.ai.umich.edu oldw-python
```

As part of a `requirements.txt`, one can install it using:

```
--extra-index-url https://packages.ai.umich.edu
# …
oldw_python==x.y.z
```

### Usage

This is a SQLAlchemy schema, so all of the querying concepts and techniques apply. To connect to the OLDW, you will need an account to connect to the Oracle system.

#### Connection string

SQLAlchemy uses URLs as connection strings. Due to the way that the Oracle host is set up at UM, using a DSN rather than a more usual `host:port` configuration is necessary. The connection string will look like:

```
oracle+oracledb://<username>:<password>@<dsn>
```

Currently the DSN for EDWPROD is:

```
(description=(SOURCE_ROUTE=yes)(ADDRESS_LIST=(FAILOVER=on)(LOAD_BALANCE=on)(ADDRESS=(PROTOCOL=tcp)(HOST=oraconnt1.dsc.umich.edu)(PORT=30421))(ADDRESS=(PROTOCOL=tcp)(HOST=oraconnt2.dsc.umich.edu)(port=30421)))(address=(protocol=tcp)(host=edwprod-db.dsc.umich.edu)(port=1521))(connect_data=(service_name=rmt_edwprod)))
```

#### Creating a SQLAlchemy session

SQLAlchemy's session manages the ORM objects for fetching and traversal. The TL;DR version is:

```python
from sqlalchemy import create_engine, select
from sqlalchemy.orm import Session

from oldw.schema import Course

from myproject.settings import OLDW_CONNECTION_STRING

engine = create_engine(OLDW_CONNECTION_STRING)
with Session(engine) as session:
    select(Course)…
```

## Development

> TBD

### Testing

> More detail to come

The test suite uses [pytest](http://pytest.org).

Install the test requirements into your virtual environment by loading `requirements-dev.txt`:

```shell
$ pip install -r requirements-dev.txt
```

In addition, an Oracle DB host is necessary to run tests. This is most easily done using Docker.

Once the DB servers are running, execute `pytest`, which should do its job.

### Versioning

This project follows [SemVer](https://semver.org). It's possible that a [CalVer](https://calver.org) scheme may be used in the future, as changes in the data warehouse schema occur outside of our control.

### Code Quality checks

This project uses `ruff` to conform to some rudimentary quality checks.
